/*
    Copyright (c) 2010, 2011 aizo ag, Zurich, Switzerland

    Author: Michael Tross <mt@aizo.ag>

    This file is part of dS-Assistant.

    dS-Assistant is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    dS-Assistant is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.

    NOTE: all APIs, especially the JSON API, offered by this application
          are subject to change without further notice.
*/

#ifndef __HWSTATS_H__
#define __HWSTATS_H__

#include <boost/shared_ptr.hpp>
#include <digitalSTROM/ds.h>
#include <inttypes.h>
#include <string>
#include <vector>
#include <list>

/// \file hwstats.h
/// \brief Hardware statistic classes.

namespace dSA
{

class StatisticBase
{
public:
    StatisticBase(uint8_t version);
    virtual ~StatisticBase();

    uint8_t getVersion() const;
    uint8_t getValue8(uint8_t index) const;
    uint16_t getValue16(uint8_t index) const;
    uint32_t getValue32(uint8_t index) const;

    void pushValue(uint8_t val);
    void pushValue(uint16_t val);
    void pushNilValue(uint8_t num = 1);

protected:
    uint8_t m_version;
    uint8_t m_index;
    uint8_t m_data[256];
    bool m_valid[256];
};

class StatisticDevice : public StatisticBase
{
public:
    StatisticDevice(uint8_t version);

    boost::shared_ptr<std::list<std::string> > getResults();
};

}
#endif
