/*
    Copyright (c) 2010, 2011 aizo ag, Zurich, Switzerland

    Author: Sergey 'Jin' Bostandzhyan <jin@dev.digitalstrom.org>

    This file is part of dS Assistant.

    dS-Assistant is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    dS-Assistant is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.

    NOTE: all APIs, especially the JSON API, offered by this application
          are subject to change without further notice.
*/

#ifdef HAVE_CONFIG_H
#include "autoconfig.h"
#endif

#include "json_dsm_web_handler.h"
#include "ds-assistant.h"
#include "exceptions.h"
#include "storage.h"

namespace dSA
{

JSONdSMWebHandler::JSONdSMWebHandler(struct mg_connection* connection,
                               const struct mg_request_info* request_info) :
                               JSONWebHandler(connection, request_info) {}

boost::shared_ptr<const dSM> JSONdSMWebHandler::getDSMfromParam()
{
    std::string dsid = m_parameters["dsid"];
    if (dsid.empty())
    {
        throw _Exception("missing dSM dSID parameter");
    }

    dsid_t dsm_dsid = util::str2dsid(dsid);


    boost::shared_ptr<const dSM> dsm = Storage::getInstance()->getDSM(dsm_dsid);

    if (dsm == NULL)
    {
        throw _Exception("dSM " + util::str(dsm_dsid) + " not found");
    }

    return dsm;
}

std::string JSONdSMWebHandler::restartRegistration()
{
    std::ostringstream result;

    boost::shared_ptr<const dSM> dsm = getDSMfromParam();

    dS_Assistant::getInstance()->dS485()->restartRegistration(dsm->getDSID());

    json_object *ret = json_ok();
    result << json_object_to_json_string(ret);
    json_object_put(ret);
    return result.str();
}

std::string JSONdSMWebHandler::getCurrentTemperature()
{
    std::ostringstream result;

    boost::shared_ptr<const dSM> dsm = getDSMfromParam();

    uint16_t t;
    dS_Assistant::getInstance()->dS485()->getDSMtemperature(dsm->getDSID(), &t);

    json_object *ret = json_ok();
    json_object *res = json_object_new_object();

    json_object_object_add(res, "temperature", json_object_new_int(t));
    json_object_object_add(ret, "data", res);

    result << json_object_to_json_string(ret);
    json_object_put(ret);
    return result.str();
}

std::string JSONdSMWebHandler::blink()
{
    std::ostringstream result;

    boost::shared_ptr<const dSM> dsm = getDSMfromParam();

    uint8_t flags;
    dS_Assistant::getInstance()->dS485()->getDisplayMode(dsm->getDSID(), &flags);
    dS_Assistant::getInstance()->dS485()->setDisplayMode(dsm->getDSID(), 0x41);
    sleep(4);
    dS_Assistant::getInstance()->dS485()->setDisplayMode(dsm->getDSID(), flags);

    json_object *ret = json_ok();
    result << json_object_to_json_string(ret);
    json_object_put(ret);
    return result.str();
}

void JSONdSMWebHandler::process()
{
    try
    {
        std::string result;
        std::string header = http_ok();

        if (m_command == "restartRegistration")
        {
            result = restartRegistration();
        }
        else if (m_command == "getCurrentTemperature")
        {
            result = getCurrentTemperature();
        }
        else if (m_command == "blink")
        {
            result = blink();
        }
        else
        {
            failure("unknown function called");
            return;
        }

        mg_write(m_connection, header.c_str(), header.length());
        mg_write(m_connection, result.c_str(), result.length());
    }
    catch (Exception ex)
    {
        failure(ex.getMessage().c_str());
    }
    catch (...)
    {
        failure("exception raised while processing call " + m_command);
    }
}

} // namespace

